#include "Arduino.h"
#include "Wire.h"

/*
   Pins - Arduino Uno - A4 (SDA), A5 (SCL) other boards see https://www.arduino.cc/en/Reference/Wire
          Arduino Nano - A4 (SCL), A5 (SDA) pins swaped from Uno.

   CatGenie 120 wire colors. This is just a starting point as they like to swap the colors. If this doesn't work you will need to probe them on your device.

   White - SDA - pin A4
   Black - SDL - pin A5
   Red - Power(5v)
   Blue - Ground

*/

byte I2C_ADDRESS = 80; // 80(7 bits) with a R/W bit appended

byte* readRegister = new byte[0x10];
int readRegisterSize;
byte remaining = 0x78;

byte* response;
int size = 4;

// Data that will be "read" from RFID tag by block id
byte BLOCK_0[] = { 0x04, 0x00, 0x00, 0x00, 0x00 };
byte BLOCK_5[] = { 0x04, 0x78, 0x00, 0x00, 0x00 };
//byte BLOCK_6[] = { 0x04, 0x78, 0x00, 0x00, 0x00 };
byte BLOCK_8[] = { 0x04, 0x01, 0x00, 0x78, 0x00 };
//byte BLOCK_9[] = { 0x04, 0x01, 0x00, 0x78, 0x00 };
//byte BLOCK_A[] = { 0x04, 0x01, 0x00, 0x78, 0x00 };
byte BLOCK_B[] = { 0x04, 0x04, 0x0F, 0x00, 0x00 };
//byte BLOCK_C[] = { 0x04, 0x04, 0x0F, 0x00, 0x00 };
byte BLOCK_D[] = { 0x04, 0x1F, 0x7E, 0x30, 0x2A };
byte BLOCK_F[] = { 0x04, 0x04, 0x0F, 0x2B, 0xFC };

byte* BLOCKS[] = {
  BLOCK_0, BLOCK_0, BLOCK_0, BLOCK_0,
  BLOCK_0, BLOCK_5, BLOCK_5, BLOCK_0,
  BLOCK_8, BLOCK_8, BLOCK_8, BLOCK_B,
  BLOCK_B, BLOCK_D, BLOCK_0, BLOCK_F
};

byte NODE_ID_RESPONSE[] = { 0x01, 0x3C };
byte UID_RESPONSE[] = { 0x08, 0xA3, 0x6E, 0x6A, 0x73, 0x09, 0x33, 0x02, 0xD0 };

void receiveEvent(int length) {
  // skip single byte write as its setting up the next read
  // and reading the ack breaks the following read
  if (length == 1) {
    // consume data but ignore
    Wire.read();
    return;
  }

  // all other instructions set the read register for a subsiquent block read (handled by requestEvent)
  for (int i = 0 ; Wire.available() ; i++) {
    readRegister[i] = Wire.read();
  }
  readRegisterSize = length;

  int command = readRegister[2] & 0x0f;

  switch (command)
  {
    case 0x06: // INITIATE, PCALL16, SLOT_MARKER
      response = NODE_ID_RESPONSE;
      size = 2;
      break;
    case 0x08: // RESET_BLOCK
      response = BLOCKS[readRegister[3]];
      size = 5;
      break;
    case 0x09: // WRITE_BLOCK
      response = BLOCKS[readRegister[3]];
      response[1] = readRegister[4];
      response[2] = readRegister[5];
      response[3] = readRegister[6];
      response[4] = readRegister[7];
      size = 0;
      break;
    case 0x0A: // AUTHENTICATE (ignored)
      size = 0;
      break;
    case 0x0B: // GET_UID
      response = UID_RESPONSE;
      size = 9;
      break;
    case 0x0C: // RESET_TO_INVENTORY
      size = 0;
      break;
    case 0x0E: // SELECT
      response = NODE_ID_RESPONSE;
      size = 2;
      break;
    case 0x0F: // COMPLETION
      size = 0;
      break;
  }
}

// WARNING - This needs to be pretty fast or we will miss data. Serial prints are too slow.
void requestEvent() {
  Wire.write(response, size);
}

void setup() {
  Wire.begin(I2C_ADDRESS);

  Wire.onRequest(requestEvent);
  Wire.onReceive(receiveEvent);
}

void loop() {
  delay(100);
}
